#include "centralwidget.h"

#include <QtGui>
#include "exceptions.h"
#include "xml/xmlreader.h"
#include "xml/xmlwriter.h"

XkorCentralWidget::XkorCentralWidget(QWidget * parent) : QWidget(parent)
{
	currentEvent = 0;
	modified = false;

	nw = new XkorNavigationWidget;
	editor = new QStackedWidget;

	// set up editor widgets
	rpe = new XkorRPEditor;
	connect(rpe, SIGNAL(dataChanged()), this, SLOT(setModified()));
	editor->addWidget(rpe);
	editor->setCurrentWidget(rpe);
	connect(nw, SIGNAL(listChanged()), this, SLOT(setModified()));
	connect(nw, SIGNAL(editRPList(XkorRPList *)), rpe, SLOT(setData(XkorRPList *)));
	connect(nw, SIGNAL(editRPList(XkorRPList *)), this, SLOT(showRPEditor()));

	ee = new XkorEventEditor;
	connect(ee, SIGNAL(dataChanged()), this, SLOT(setModified()));
	editor->addWidget(ee);
	connect(nw, SIGNAL(editEvent(XkorEvent *)), this, SLOT(showEventEditor(XkorEvent *)));

	QSplitter * splitter = new QSplitter;
	splitter->addWidget(nw);
	splitter->addWidget(editor);

	QGridLayout * layout = new QGridLayout(this);
	layout->addWidget(splitter);
}

void XkorCentralWidget::closeEvent(QCloseEvent * event)
{
	if(okayToLoad())
		QWidget::closeEvent(event);
	else
		event->ignore();
}

QList<QPair<QUuid, XkorEvent *> > XkorCentralWidget::events()
{
	updateCurrentEvent();
	return nw->events();
}

void XkorCentralWidget::newFile()
{
	if(okayToLoad())
	{
		XkorRPList * rpl = new XkorRPList;
		QList<QPair<QUuid, XkorEvent *> > events;

		rpe->setData(NULL);
		ee->setData(XkorEvent(), XkorRPList());
		nw->reset();
		currentEvent = 0;

		nw->setRPList(rpl);
		nw->setEvents(events);

		editor->setCurrentWidget(rpe);

		currentFileName = "";
		emit fileChanged(currentFileName);
		setModified(false);
	}
}

bool XkorCentralWidget::okayToLoad()
{
	if(modified)
	{
		int result = showUnsavedDialog();
		if(result == QMessageBox::Yes)
			saveFile();
		else if(result == QMessageBox::Cancel)
			return false;
	}
	return true;
}

void XkorCentralWidget::openFile()
{
	QString filename;
	if(okayToLoad())
		filename = QFileDialog::getOpenFileName(this, tr("Open scorinator file"), ".", tr("XML files (*.xml)"));
	if(filename != "")
	{
		XkorXmlReader r(filename);
		if(r.hasError())
			QMessageBox::critical(this, tr("xkoranate"), r.error());
		else
		{
			XkorRPList * rpl = r.rpList();
			QList<QPair<QUuid, XkorEvent *> > events = r.events();

			rpe->setData(NULL);
			ee->setData(XkorEvent(), XkorRPList());
			nw->reset();
			currentEvent = 0;

			nw->setRPList(rpl);
			nw->setEvents(events);

			editor->setCurrentWidget(rpe);

			currentFileName = filename;

			emit fileChanged(filename);
			setModified(false);
		}
	}
}

XkorRPList * XkorCentralWidget::rpList()
{
	rpe->updateData();
	qDebug() << nw->rpList()->rpEffect();
	return nw->rpList();
}

void XkorCentralWidget::saveFile(QString filename)
{
	if(filename == QString()) // if no filename was specified
	{
		if(currentFileName == QString()) // still “Untitled”?
			saveFileAs();
		else
			filename = currentFileName;
	}
	QFile f(filename);

	XkorXmlWriter w(filename, rpList(), events());

	currentFileName = filename;
	emit fileChanged(filename);
	setModified(false);
}

void XkorCentralWidget::saveFileAs()
{
	QString filename = QFileDialog::getSaveFileName(this, tr("Save scorinator file"), ".", tr("XML files (*.xml)"));
	if(filename != QString())
		saveFile(filename);
}

void XkorCentralWidget::setModified(bool isModified)
{
	modified = isModified;
	if(isModified)
		emit fileEdited();
}

void XkorCentralWidget::showEventEditor(XkorEvent * e)
{
	updateCurrentEvent();
	currentEvent = e;
	ee->setData(*e, *rpList());
	editor->setCurrentWidget(ee);
}

void XkorCentralWidget::showRPEditor()
{
	editor->setCurrentWidget(rpe);
}

int XkorCentralWidget::showUnsavedDialog()
{
	return QMessageBox::warning(this, tr("xkoranate"), tr("The document has been modified.\nDo you want to save your changes?"), QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel, QMessageBox::Cancel);
}

void XkorCentralWidget::updateCurrentEvent()
{
	if(currentEvent)
		*currentEvent = ee->data();
}
