#include "competitions/roundrobincompetition.h"

#include <QTextCodec>
#include <algorithm>
#include "competitions/options/roundrobincompetitionoptions.h"
#include "paradigms/paradigmfactory.h"

XkorAbstractOptionsWidget * XkorRoundRobinCompetition::newOptionsWidget(QHash<QString, QVariant> competitionOptions)
{
	return new XkorRoundRobinCompetitionOptions(competitionOptions);
}

void XkorRoundRobinCompetition::scorinate(int matchday)
{
	if(largestGroupSize(startList) > 0) // if there are no teams, don’t bother!
	{
		// initialize the tables
		tables.clear();
		std::vector<QString> sortCriteria = tableSortCriteria();
		unsigned int groupNo = 0;
		for(std::vector<XkorStartListGroup>::iterator i = startList.groups.begin(); i != startList.groups.end(); ++i)
		{
			XkorTable t;
			t.setColumns(generateTableColumns(i->name));
			t.setMatches(retrieveMatches(groupNo));
			t.setSortCriteria(sortCriteria);
			t.setPointsForWin(userOpt.contains("pointsForWin") ? userOpt.value("pointsForWin").toInt() : 3);
			t.setPointsForDraw(userOpt.contains("pointsForDraw") ? userOpt.value("pointsForDraw").toInt() : 1);
			t.setPointsForLoss(userOpt.contains("pointsForLoss") ? userOpt.value("pointsForLoss").toInt() : 0);
			tables.append(t);
			++groupNo;
		}

		// initialize the results in the resume file options
		resumeOpt.insert("tableData", userOpt.value("tableData"));
		
		unsigned int numberOfLegs = 1;
		if(userOpt.contains("numberOfLegs"))
			numberOfLegs = userOpt.value("numberOfLegs").toUInt();

		scorinateMatchday(matchday);
		resultsBuf[matchday] += "\n";
	}
}

QList<std::pair<unsigned int, unsigned int> > XkorRoundRobinCompetition::generateFixtures(unsigned int matchday, unsigned int groupSize)
{
	// account for possibility of more than two legs
	bool reverseFixtures = (matchday % (2 * (groupSize - 1)) >= groupSize - 1); // if we’re in the 2nd, 4th, … leg, reverse home/away
	matchday %= (groupSize - 1);
	
	unsigned int matchesPerRound = groupSize / 2;
	QList<std::pair<unsigned int, unsigned int> > fixtures;
	
	// mix up the fixtures a bit
	unsigned int round = matchday / 2;
	if(matchday & 1)
		round += groupSize / 2;
	
	for(unsigned int m = 0; m < matchesPerRound; ++m)
	{
		unsigned int home = (round + m) % (groupSize - 1);
		unsigned int away = (groupSize - 1 - m + round) % (groupSize - 1);
		if(m == 0)
			away = groupSize - 1;
		
		// flip the fixtures if:
		// if this is the first match of an odd matchday, flip so that the last team isn’t always away
		// if we’re on an even-numbered leg (reverseFixtures is set)
		// if both of these are true, “flip it twice,” i.e., don’t flip it (thus use of xor)
		if((m == 0 && (matchday & 1)) xor reverseFixtures)
			fixtures.append(make_pair(away, home));
		else
			fixtures.append(make_pair(home, away));
	}

	return fixtures;
}

std::vector<XkorTableColumn> XkorRoundRobinCompetition::generateTableColumns(QString groupName)
{
	std::vector<XkorTableColumn> columns;
	
	if(userOpt.contains("tableColumns"))
	{
		QList<QVariant> columnTypes = userOpt.value("tableColumns").toList();
		QList<QVariant> columnHeadings = userOpt.value("tableHeadings").toList();
		QList<QVariant> columnWidths = userOpt.value("tableWidths").toList();
		
		for(int i = 0; i < columnTypes.size() && i < columnHeadings.size() && i < columnWidths.size(); ++i)
			if(columnTypes[i] == "name")
				columns.push_back(XkorTableColumn(columnTypes[i].toString(), groupName, columnWidths[i].toUInt()));
			else
				columns.push_back(XkorTableColumn(columnTypes[i].toString(), columnHeadings[i].toString(), columnWidths[i].toUInt()));
	}
	else
		columns = XkorTableColumnsWidget::defaultColumns(groupName);
	return columns;
}

unsigned int XkorRoundRobinCompetition::largestGroupSize(XkorStartList & sl)
{
	int rval = 0;
	for(std::vector<XkorStartListGroup>::iterator i = sl.groups.begin(); i != sl.groups.end(); ++i)
		if(i->athletes.size() > rval)
			rval = i->athletes.size();
	if(rval & 1)
		rval += 1; // rval should be even
	return rval;
}

int XkorRoundRobinCompetition::matchdays()
{
	if(userOpt.contains("numberOfLegs"))
		return userOpt.value("numberOfLegs").toInt() * (largestGroupSize(startList) - 1);
	else
		return largestGroupSize(startList) - 1;
}

QStringList XkorRoundRobinCompetition::matchdayNames()
{
	QStringList rval;
	for(int i = 0; i < matchdays(); ++i)
		rval << QString("Matchday ") + QString::number(i + 1);
	return rval;
}

std::vector<XkorTableMatch> XkorRoundRobinCompetition::retrieveMatches(unsigned int groupNo)
{
	std::vector<XkorTableMatch> rval;
	
	if(userOpt.contains("tableData"))
	{
		QList<QVariant> tableData = userOpt.value("tableData").toList();
		if((unsigned int)tableData.size() > groupNo)
		{
			QList<QVariant> results = tableData[groupNo].toList();
			for(QList<QVariant>::iterator i = results.begin(); i != results.end(); ++i)
			{
				QRegExp rx("[0-9]+: (.+) ([0-9]+)–([0-9]+) (.+)"); // match scores of form “1: Aquilla 3–1 Busby”
				if(rx.indexIn(i->toString()) != -1) // if we matched
					rval.push_back(XkorTableMatch(rx.cap(1), rx.cap(4), rx.cap(2).toDouble(), rx.cap(3).toDouble()));
			}
		}
	}
	return rval;
}

QHash<QString, QVariant> XkorRoundRobinCompetition::revertToMatchday(int matchday)
{
	// delete the results
	for(QHash<int, QString>::iterator i = resultsBuf.begin(); i != resultsBuf.end(); ++i)
		if(i.key() >= matchday)
			i.value() = "";

	// sort out the table data
	QList<QVariant> tableData = userOpt.value("tableData").toList();
	for(QList<QVariant>::iterator i = tableData.begin(); i != tableData.end(); ++i)
	{
		QList<QVariant> groupData = i->toList();
		for(int j = 0; j < groupData.size(); ++j)
		{
			QRegExp rx("([0-9]+): (.+ [0-9]+–[0-9]+ .+)"); // find out what matchday the score is from
			if(rx.indexIn(groupData[j].toString()) != -1 && rx.cap(1).toDouble() >= matchday) // if it’s on “matchday” or later…
				groupData[j] = ""; // …blank the value
		}
		// actually delete the blanked values
		QList<QVariant>::iterator newEnd = remove(groupData.begin(), groupData.end(), "");
		groupData.erase(newEnd, groupData.end());
		*i = groupData;
	}
	resumeOpt["tableData"] = tableData;
	return resumeOpt;
}

void XkorRoundRobinCompetition::scorinateMatchday(unsigned int matchday)
{
	XkorAbstractParadigm * p = XkorParadigmFactory::newParadigm(&sport, paradigmOpt);

	QList<std::pair<unsigned int, unsigned int> > fixtures = generateFixtures(matchday, largestGroupSize(startList));
	
	QList<QVariant> tableData = resumeOpt.value("tableData").toList();

	// make sure we have a place to insert matches to
	while((unsigned int)(tableData.size()) < startList.groups.size())
		tableData.append(QVariant(QList<QVariant>()));

	unsigned int groupNo = 0;
	for(std::vector<XkorStartListGroup>::iterator i = startList.groups.begin(); i != startList.groups.end(); ++i)
	{
		QList<XkorAthlete> athletes;
		QList<QVariant> groupData = tableData[groupNo].toList();
		unsigned int groupSize = i->athletes.size();
		
		for(QList<std::pair<unsigned int, unsigned int> >::iterator j = fixtures.begin(); j != fixtures.end(); ++j)
			if(j->first < groupSize && j->second < groupSize)
			{
				athletes.append(i->athletes[j->first]);
				athletes.append(i->athletes[j->second]);
			}
		
		p->scorinate(athletes);
		resultsBuf[matchday] += i->name + "\n" + p->output() + "\n";
		
		// assemble teh tablez
		for(QList<std::pair<unsigned int, unsigned int> >::iterator j = fixtures.begin(); j != fixtures.end(); ++j)
			if(j->first < groupSize && j->second < groupSize)
			{
				XkorResult score1 = *(p->findResult(i->athletes[j->first].id));
				XkorResult score2 = *(p->findResult(i->athletes[j->second].id));
				tables[groupNo].insertMatch(i->athletes[j->first].name, i->athletes[j->second].name, score1.score(), score2.score());

				// insert into the resume options
				groupData.append(QString::number(matchday) + ": " + i->athletes[j->first].name + " " + QString::number(score1.score()) + "–" + QString::number(score2.score()) + " " + i->athletes[j->second].name);
			}
		
		tables[groupNo].generate();
		tableData[groupNo] = groupData;
		resultsBuf[matchday] += tables[groupNo].toText();
		resultsBuf[matchday] += "\n";
		
		++groupNo;
	}
	resumeOpt["tableData"] = tableData;

	delete p;
}

std::vector<QString> XkorRoundRobinCompetition::tableSortCriteria()
{
	std::vector<QString> rval;
	
	if(userOpt.contains("sortCriteria"))
	{
		QList<QVariant> criteria = userOpt.value("sortCriteria").toList();
		for(QList<QVariant>::iterator i = criteria.begin(); i != criteria.end(); ++i)
			rval.push_back(i->toString());
	}
	else
		rval = XkorSortCriteriaWidget::defaultSortCriteria();
	return rval;
}
